import React, { forwardRef, useLayoutEffect, createContext, useMemo, useState, useContext, useRef, useCallback, useEffect } from 'react';
import classNames from 'classnames';
import { transparentize, readableColor } from 'polished';
import { createStyles, ThemeProvider as ThemeProvider$1, ColorContextProvider, css, useTheme, useColorContext, createReadablePalette } from 'react-style-system';
export { ColorContextProvider, ThemeContext, createStyles, css, useColorContext, useTheme } from 'react-style-system';
import shortId from 'shortid';
import { createPortal } from 'react-dom';

/*! *****************************************************************************
Copyright (c) Microsoft Corporation. All rights reserved.
Licensed under the Apache License, Version 2.0 (the "License"); you may not use
this file except in compliance with the License. You may obtain a copy of the
License at http://www.apache.org/licenses/LICENSE-2.0

THIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED
WARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,
MERCHANTABLITY OR NON-INFRINGEMENT.

See the Apache Version 2.0 License for specific language governing permissions
and limitations under the License.
***************************************************************************** */

function __rest(s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
}

function __awaiter(thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
}

const useStyles = createStyles(({ css, theme, color }) => ({
    // button base styles
    root: css `
    ${theme.fonts.button}

    display: inline-flex;
    justify-content: center;
    align-items: center;

    appearance: none;
    outline: none;
    background: transparent;
    border: none;
    text-decoration: none;
    cursor: pointer;

    &:disabled {
      cursor: not-allowed;
    }
  `,
    // shapes
    standard: css `
    min-width: ${theme.block(1)};

    & > *:not(:last-child) {
      margin-right: ${theme.space(1)};
    }
  `,
    icon: css `
    border-radius: 99999px;
    & > svg {
      width: ${theme.space(1)};
      height: ${theme.space(1)};
    }
  `,
    // variants
    outlined: css `
    border: 1px solid ${color.readable};
    color: ${color.readable};

    transition: background-color ${theme.durations.standard}ms,
      border ${theme.durations.standard}ms;

    &:focus {
      background-color: ${transparentize(0.92, color.decorative)};
    }
    &:hover {
      background-color: ${transparentize(0.9, color.decorative)};
    }
    &:active {
      background-color: ${transparentize(0.8, color.decorative)};
    }
    &:disabled {
      color: ${transparentize(0.4, color.readable)};
      background-color: transparent;
    }
  `,
    filled: css `
    background-color: ${color.decorative};
    color: ${readableColor(color.decorative)};
    border: 2px solid transparent;
    transition: background-color ${theme.durations.standard}ms,
      border ${theme.durations.standard}ms;

    &:focus {
      background-color: ${transparentize(0.18, color.decorative)};
      border: 2px solid ${transparentize(0.7, '#fff')};
    }
    &:hover {
      background-color: ${transparentize(0.23, color.decorative)};
    }
    &:active {
      background-color: ${transparentize(0.3, color.decorative)};
      border: 2px solid ${transparentize(0.5, '#fff')};
    }
    &:disabled {
      background-color: ${transparentize(0.5, color.decorative)};
      border: 2px solid transparent;
      color: ${transparentize(0.2, readableColor(color.decorative))};
    }
  `,
    ghost: css `
    color: ${color.readable};
    border: 2px solid transparent;
    transition: background-color ${theme.durations.standard}ms,
      border ${theme.durations.standard}ms;

    &:focus {
      background-color: ${transparentize(0.92, color.decorative)};
      border: 2px solid ${transparentize(0.8, readableColor(color.decorative))};
    }
    &:hover {
      background-color: ${transparentize(0.9, color.decorative)};
    }
    &:active {
      background-color: ${transparentize(0.8, color.decorative)};
    }
    &:disabled {
      color: ${transparentize(0.4, color.readable)};
      background-color: transparent;
    }
  `,
    // sizes
    standardSmall: css `
    padding: ${theme.space(0.5)} ${theme.space(0.75)};
    min-width: ${theme.block(1)};
  `,
    standardStandard: css `
    padding: ${theme.space(0.75)} ${theme.space(1)};
    min-width: ${theme.block(1)};
  `,
    standardLarge: css `
    padding: ${theme.space(1)} ${theme.space(1.25)};
    min-width: ${theme.block(1.5)};
  `,
    iconSmall: css `
    width: ${theme.space(2)};
    height: ${theme.space(2)};
  `,
    iconStandard: css `
    width: ${theme.space(2.5)};
    height: ${theme.space(2.5)};
  `,
    iconLarge: css `
    width: ${theme.space(3)};
    height: ${theme.space(3)};
  `,
}));
const Button = forwardRef((props, ref) => {
    const _a = useStyles(props, props.component || 'button'), { Root, styles, component, variant = 'ghost', size = 'standard', shape = 'standard' } = _a, restOfProps = __rest(_a, ["Root", "styles", "component", "variant", "size", "shape"]);
    return (React.createElement(Root, Object.assign({ className: classNames('hui-button', {
            [styles.icon]: shape === 'icon',
            [styles.standard]: shape === 'standard',
            [styles.outlined]: variant === 'outlined',
            [styles.ghost]: variant === 'ghost',
            [styles.filled]: variant === 'filled',
            [styles.iconSmall]: shape === 'icon' && size === 'small',
            [styles.iconStandard]: shape === 'icon' && size === 'standard',
            [styles.iconLarge]: shape === 'icon' && size === 'large',
            [styles.standardSmall]: shape === 'standard' && size === 'small',
            [styles.standardStandard]: shape === 'standard' && size === 'standard',
            [styles.standardLarge]: shape === 'standard' && size === 'large',
        }), ref: ref }, restOfProps)));
});

const useStyles$1 = createStyles(({ css }) => ({
    root: css `
    list-style-type: none;
    padding: 0;
    margin: 0;
  `,
}));
const List = forwardRef((props, ref) => {
    const _a = useStyles$1(props, 'ul' ), { Root, styles } = _a, restOfProps = __rest(_a, ["Root", "styles"]);
    return React.createElement(Root, Object.assign({ ref: ref }, restOfProps));
});

const useStyles$2 = createStyles(({ css, theme }) => ({
    root: css `
    ${theme.fonts.body1};
    display: flex;
  `,
}));
const ListItem = forwardRef((props, ref) => {
    const _a = useStyles$2(props, 'li'), { Root, styles } = _a, restOfProps = __rest(_a, ["Root", "styles"]);
    return React.createElement(Root, Object.assign({ ref: ref }, restOfProps));
});

function ThemeProvider({ theme, children }) {
    return (React.createElement(ThemeProvider$1, { theme: theme },
        React.createElement(ColorContextProvider, { color: theme.colors.accent, surface: theme.colors.surface }, children)));
}

const defaultTheme = {
    fonts: {
        button: css `
      font-size: 0.75rem;
      font-weight: bold;
      text-transform: uppercase;
      margin: 0;
    `,
        caption: css `
      font-size: 0.8rem;
      margin: 0;
    `,
        h1: css `
      font-size: 160px;
      font-weight: bold;
      margin: 0;
    `,
        h2: css `
      font-size: 100px;
      font-weight: bold;
      margin: 0;
    `,
        h3: css `
      font-size: 60px;
      font-weight: bold;
      margin: 0;
    `,
        h4: css `
      font-size: 24px;
      font-weight: bold;
      line-height: 1.5;
      margin: 0;
    `,
        h5: css `
      font-size: 16px;
      font-weight: bold;
      line-height: 1.5;
      margin: 0;
    `,
        subtitle1: css `
      font-size: 60px;
      font-weight: bold;
      color: #444;
      margin: 0;
    `,
        subtitle2: css `
      font-size: 24px;
      font-weight: bold;
      color: #444;
      margin: 0;
    `,
        body1: css `
      font-weight: 400;
      text-transform: none;
      font-size: 16px;
      margin: 0;
      line-height: 1.5;
    `,
        body2: css `
      font-weight: 400;
      text-transform: none;
      font-size: 16px;
      margin: 0;
      color: #444;
    `,
    },
    colors: {
        brand: '#000',
        accent: '#2962ff',
        bland: '#ccc',
        danger: '#eb002b',
        warning: '#f56200',
        info: '#2962ff',
        surface: '#fff',
    },
    durations: {
        short: 100,
        standard: 250,
        long: 500,
    },
    shadows: {
        subtle: '0 0 10px 0 rgba(0,0,0,0.10)',
        standard: '0 0 20px 0 rgba(0,0,0,0.13)',
        emphasis: '0 0 30px 0 rgba(0,0,0,0.13)',
    },
    space: n => `${n * 16}px`,
    gap: n => `${n * 48}px`,
    block: n => `${n * 96}px`,
    zIndex: {
        appBar: 1100,
        drawer: 1200,
        modal: 1300,
        notification: 1400,
        tooltip: 1500,
    },
    breakpoints: {
        mobile: 375,
        tablet: 768,
        desktop: 1024,
        desktopLarge: 1440,
    },
};

const normalize = `
/*! normalize.css v8.0.1 | MIT License | github.com/necolas/normalize.css */html{line-height:1.15;-webkit-text-size-adjust:100%}body{margin:0}main{display:block}h1{font-size:2em;margin:.67em 0}hr{box-sizing:content-box;height:0;overflow:visible}pre{font-family:monospace,monospace;font-size:1em}a{background-color:transparent}abbr[title]{border-bottom:none;text-decoration:underline;text-decoration:underline dotted}b,strong{font-weight:bolder}code,kbd,samp{font-family:monospace,monospace;font-size:1em}small{font-size:80%}sub,sup{font-size:75%;line-height:0;position:relative;vertical-align:baseline}sub{bottom:-.25em}sup{top:-.5em}img{border-style:none}button,input,optgroup,select,textarea{font-family:inherit;font-size:100%;line-height:1.15;margin:0}button,input{overflow:visible}button,select{text-transform:none}[type=button],[type=reset],[type=submit],button{-webkit-appearance:button}[type=button]::-moz-focus-inner,[type=reset]::-moz-focus-inner,[type=submit]::-moz-focus-inner,button::-moz-focus-inner{border-style:none;padding:0}[type=button]:-moz-focusring,[type=reset]:-moz-focusring,[type=submit]:-moz-focusring,button:-moz-focusring{outline:1px dotted ButtonText}fieldset{padding:.35em .75em .625em}legend{box-sizing:border-box;color:inherit;display:table;max-width:100%;padding:0;white-space:normal}progress{vertical-align:baseline}textarea{overflow:auto}[type=checkbox],[type=radio]{box-sizing:border-box;padding:0}[type=number]::-webkit-inner-spin-button,[type=number]::-webkit-outer-spin-button{height:auto}[type=search]{-webkit-appearance:textfield;outline-offset:-2px}[type=search]::-webkit-search-decoration{-webkit-appearance:none}::-webkit-file-upload-button{-webkit-appearance:button;font:inherit}details{display:block}summary{display:list-item}template{display:none}[hidden]{display:none}
body {font-family: -apple-system, BlinkMacSystemFont, "Segoe UI", Roboto, Helvetica, Arial, sans-serif, "Apple Color Emoji", "Segoe UI Emoji", "Segoe UI Symbol";}
:root{font-size:16px}
* { box-sizing: border-box; }
`;
function useCssReset() {
    useLayoutEffect(() => {
        if (document.querySelector('#hacker-ui-css-reset')) {
            return;
        }
        const styleEl = document.createElement('style');
        styleEl.innerHTML = normalize;
        styleEl.id = 'hacker-ui-css-reset';
        document.head.appendChild(styleEl);
    }, []);
}

function isObject(maybe) {
    if (!maybe)
        return false;
    return typeof maybe === 'object';
}
/**
 * recursively merges a base object with an incoming object
 */
function recursiveMerge(base, incoming = {}) {
    const keys = [...Object.keys(base), ...Object.keys(incoming)];
    const merged = keys.reduce((merged, key) => {
        const baseValue = base[key];
        const incomingValue = incoming[key];
        const mergedValue = isObject(baseValue)
            ? recursiveMerge(baseValue, incomingValue)
            : (incomingValue || baseValue);
        merged[key] = mergedValue;
        return merged;
    }, {});
    return merged;
}
function createTheme(partialTheme = {}) {
    return recursiveMerge(defaultTheme, partialTheme);
}

const useStyles$3 = createStyles(({ css, theme }) => ({
    root: css `
    ${theme.fonts.body1}
    align-items: flex-start;
    width: 100%;
    display: flex;
    flex-direction: column;
  `,
}));
const ListItemButton = forwardRef((props, ref) => {
    const theme = useTheme();
    const _a = useStyles$3(props, Button), { Root, styles } = _a, restOfProps = __rest(_a, ["Root", "styles"]);
    return (React.createElement(Root, Object.assign({ ref: ref, variant: "ghost", color: theme.colors.bland }, restOfProps)));
});

const useStyles$4 = createStyles(({ css, theme, color }) => ({
    root: css `
    color: ${color.readable};
    text-decoration: underline;
    transition: color ${theme.durations.standard}ms;

    &:active {
      color: ${transparentize(0.5, color.readable)};
    }
  `,
}));
const Anchor = forwardRef((props, ref) => {
    var _a;
    const _b = useStyles$4(props, (_a = props.component, (_a !== null && _a !== void 0 ? _a : 'a'))), { Root, styles, component } = _b, restOfProps = __rest(_b, ["Root", "styles", "component"]);
    return React.createElement(Root, Object.assign({ ref: ref }, restOfProps));
});

var FormControlContext = createContext(null);

const useStyles$5 = createStyles(({ css }) => ({
    root: css `
    display: flex;
    flex-direction: column;
  `,
}));
const FormControl = forwardRef((props, ref) => {
    var _a, _b, _c, _d, _e, _f;
    const _g = useStyles$5(props, props.component || 'div'), { Root, styles, component, hasError = false, disabled = false } = _g, restOfProps = __rest(_g, ["Root", "styles", "component", "hasError", "disabled"]);
    const colorContext = useColorContext();
    const theme = useTheme();
    const color = (_c = (_a = props.color, (_a !== null && _a !== void 0 ? _a : (_b = colorContext) === null || _b === void 0 ? void 0 : _b.color)), (_c !== null && _c !== void 0 ? _c : theme.colors.accent));
    const surface = (_f = (_d = props.surface, (_d !== null && _d !== void 0 ? _d : (_e = colorContext) === null || _e === void 0 ? void 0 : _e.surface)), (_f !== null && _f !== void 0 ? _f : theme.colors.surface));
    const id = useMemo(() => `hui-${shortId()}`, []);
    const [focused, setFocused] = useState(false);
    const formControlContextValue = useMemo(() => ({
        id,
        focused,
        setFocused,
        hasError,
        disabled,
    }), [disabled, focused, hasError, id]);
    return (React.createElement(ColorContextProvider, { color: color, surface: surface },
        React.createElement(FormControlContext.Provider, { value: formControlContextValue },
            React.createElement(Root, Object.assign({ ref: ref }, restOfProps)))));
});

const useStyles$6 = createStyles(({ css, theme, color, surface }) => {
    const bland = createReadablePalette(theme.colors.bland, surface);
    return {
        root: css `
      ${theme.fonts.body1};
      color: ${readableColor(theme.colors.surface)};
      transition: color ${theme.durations.standard}ms;
      cursor: pointer;
    `,
        focused: css `
      color: ${color.readable};
    `,
        hasError: css `
      color: ${theme.colors.danger};
    `,
        disabled: css `
      color: ${transparentize(0.3, bland.readable)};
      cursor: not-allowed;
    `,
    };
});
const Label = forwardRef((props, ref) => {
    var _a, _b, _c, _d, _e, _f, _g;
    const _h = useStyles$6(props, props.component || 'label'), { Root, styles, component, id: incomingId, focused: incomingFocused, hasError: incomingHasError, disabled: incomingDisabled } = _h, restOfProps = __rest(_h, ["Root", "styles", "component", "id", "focused", "hasError", "disabled"]);
    const formControlContext = useContext(FormControlContext);
    const id = (incomingId !== null && incomingId !== void 0 ? incomingId : (_a = formControlContext) === null || _a === void 0 ? void 0 : _a.id);
    const focused = (_c = (incomingFocused !== null && incomingFocused !== void 0 ? incomingFocused : (_b = formControlContext) === null || _b === void 0 ? void 0 : _b.focused), (_c !== null && _c !== void 0 ? _c : false));
    const hasError = (_e = (incomingHasError !== null && incomingHasError !== void 0 ? incomingHasError : (_d = formControlContext) === null || _d === void 0 ? void 0 : _d.hasError), (_e !== null && _e !== void 0 ? _e : false));
    const disabled = (_g = (incomingDisabled !== null && incomingDisabled !== void 0 ? incomingDisabled : (_f = formControlContext) === null || _f === void 0 ? void 0 : _f.disabled), (_g !== null && _g !== void 0 ? _g : false));
    return (React.createElement(Root, Object.assign({ className: classNames({
            [styles.focused]: focused,
            [styles.hasError]: hasError,
            [styles.disabled]: disabled,
        }), ref: ref, htmlFor: id }, restOfProps)));
});

const useStyles$7 = createStyles(({ css, theme, color, surface }) => {
    const bland = createReadablePalette(theme.colors.bland, surface);
    const danger = createReadablePalette(theme.colors.danger, surface);
    return {
        root: css `
      ${theme.fonts.body1};
      padding: ${theme.space(0.75)} ${theme.space(0.5)};
      border: none;
      outline: none;
      appearance: none;
      background: none;
      margin: ${theme.space(0.5)} 0;

      &:disabled {
        cursor: not-allowed;
      }
    `,
        filled: css `
      background-color: ${transparentize(0.8, bland.decorative)};
      color: ${readableColor(surface)};
      transition: background-color ${theme.durations.standard}ms;

      &:focus {
        background-color: ${transparentize(0.92, color.decorative)};
        color: ${readableColor(surface)};
      }
      &:hover {
        background-color: ${transparentize(0.9, color.decorative)};
        color: ${readableColor(surface)};
      }
      &:disabled {
        background-color: ${transparentize(0.9, bland.decorative)};
      }
    `,
        filledHasError: css `
      background-color: ${transparentize(0.9, danger.decorative)};
      color: ${readableColor(surface)};
      &:focus {
        background-color: ${transparentize(0.85, danger.decorative)};
      }
      &:not([disabled]):hover {
        background-color: ${transparentize(0.87, danger.decorative)};
      }
    `,
        outlined: css `
      background-color: ${surface};
      transition: border ${theme.durations.standard}ms,
        background-color ${theme.durations.standard}ms;
      border: 2px solid ${bland.decorative};

      &:focus {
        border: 2px solid ${color.decorative};
        background-color: ${transparentize(0.93, color.decorative)};
      }
      &:hover {
        border: 2px solid ${transparentize(0.3, color.decorative)};
      }
      &:disabled {
        border: 2px solid ${transparentize(0.7, bland.decorative)};
        background-color: ${transparentize(0.9, bland.decorative)};
      }
    `,
        outlinedHasError: css `
      border: 2px solid ${danger.decorative};
      &:focus {
        border: 2px solid ${danger.decorative};
        background-color: ${transparentize(0.93, danger.decorative)};
      }
      &:not([disabled]):hover {
        border: 2px solid ${transparentize(0.3, danger.decorative)};
      }
    `,
    };
});
const TextInput = forwardRef((props, ref) => {
    var _a, _b, _c, _d;
    const _e = useStyles$7(props, (_a = props.component, (_a !== null && _a !== void 0 ? _a : 'input'))), { Root, styles, variant = 'outlined', hasError: incomingHasError, disabled: incomingDisabled, onFocus, onBlur, type = 'text' } = _e, restOfProps = __rest(_e, ["Root", "styles", "variant", "hasError", "disabled", "onFocus", "onBlur", "type"]);
    const formControlContext = useContext(FormControlContext);
    const id = (_b = formControlContext) === null || _b === void 0 ? void 0 : _b.id;
    const disabled = Boolean((_c = formControlContext) === null || _c === void 0 ? void 0 : _c.disabled) || Boolean(incomingDisabled);
    const hasError = Boolean((_d = formControlContext) === null || _d === void 0 ? void 0 : _d.hasError) || Boolean(incomingHasError);
    const handleFocus = (e) => {
        if (onFocus) {
            onFocus(e);
        }
        if (formControlContext) {
            formControlContext.setFocused(true);
        }
    };
    const handleBlur = (e) => {
        if (onBlur) {
            onBlur(e);
        }
        if (formControlContext) {
            formControlContext.setFocused(false);
        }
    };
    return (React.createElement(Root, Object.assign({ ref: ref, id: id, type: type, disabled: disabled, className: classNames({
            [styles.outlined]: variant === 'outlined',
            [styles.outlinedHasError]: variant === 'outlined' && hasError,
            [styles.filled]: variant === 'filled',
            [styles.filledHasError]: variant === 'filled' && hasError,
        }), onFocus: handleFocus, onBlur: handleBlur }, restOfProps)));
});

const useStyles$8 = createStyles(({ css, color, theme, surface }) => {
    const bland = createReadablePalette(theme.colors.bland, surface);
    return {
        root: css `
      ${theme.fonts.caption};
      transition: color ${theme.durations.standard}ms;
    `,
        focused: css `
      color: ${color.readable};
    `,
        hasError: css `
      color: ${theme.colors.danger};
    `,
        disabled: css `
      color: ${transparentize(0.3, bland.readable)};
      cursor: not-allowed;
    `,
    };
});
const HelperText = forwardRef((props, ref) => {
    var _a, _b, _c, _d, _e, _f, _g;
    const formControlContext = useContext(FormControlContext);
    const _h = useStyles$8(props, (_a = props.component, (_a !== null && _a !== void 0 ? _a : 'div'))), { Root, styles, component, focused: incomingFocused, hasError: incomingHasError, disabled: incomingDisabled } = _h, restOfProps = __rest(_h, ["Root", "styles", "component", "focused", "hasError", "disabled"]);
    const focused = (_c = (incomingFocused !== null && incomingFocused !== void 0 ? incomingFocused : (_b = formControlContext) === null || _b === void 0 ? void 0 : _b.focused), (_c !== null && _c !== void 0 ? _c : false));
    const hasError = (_e = (incomingHasError !== null && incomingHasError !== void 0 ? incomingHasError : (_d = formControlContext) === null || _d === void 0 ? void 0 : _d.hasError), (_e !== null && _e !== void 0 ? _e : false));
    const disabled = (_g = (incomingDisabled !== null && incomingDisabled !== void 0 ? incomingDisabled : (_f = formControlContext) === null || _f === void 0 ? void 0 : _f.disabled), (_g !== null && _g !== void 0 ? _g : false));
    return (React.createElement(Root, Object.assign({ className: classNames({
            [styles.focused]: focused,
            [styles.hasError]: hasError,
            [styles.disabled]: disabled,
        }), ref: ref }, restOfProps)));
});

const useStyles$9 = createStyles(({ css, theme, color, surface }) => {
    const bland = createReadablePalette(theme.colors.bland, surface);
    const danger = createReadablePalette(theme.colors.danger, surface);
    return {
        root: css `
      ${theme.fonts.body1};
      padding: ${theme.space(0.75)} ${theme.space(0.5)};
      border: none;
      outline: none;
      appearance: none;
      background: none;
      margin: ${theme.space(0.5)} 0;

      &:disabled {
        cursor: not-allowed;
      }
    `,
        filled: css `
      background-color: ${transparentize(0.8, bland.decorative)};
      color: ${readableColor(surface)};
      transition: background-color ${theme.durations.standard}ms;

      &:focus {
        background-color: ${transparentize(0.92, color.decorative)};
        color: ${readableColor(surface)};
      }
      &:hover {
        background-color: ${transparentize(0.9, color.decorative)};
        color: ${readableColor(surface)};
      }
      &:disabled {
        background-color: ${transparentize(0.9, bland.decorative)};
      }
    `,
        filledHasError: css `
      background-color: ${transparentize(0.9, danger.decorative)};
      color: ${readableColor(surface)};
      &:focus {
        background-color: ${transparentize(0.85, danger.decorative)};
      }
      &:not([disabled]):hover {
        background-color: ${transparentize(0.87, danger.decorative)};
      }
    `,
        outlined: css `
      background-color: ${surface};
      transition: border ${theme.durations.standard}ms,
        background-color ${theme.durations.standard}ms;
      border: 2px solid ${bland.decorative};

      &:focus {
        border: 2px solid ${color.decorative};
        background-color: ${transparentize(0.93, color.decorative)};
      }
      &:hover {
        border: 2px solid ${transparentize(0.3, color.decorative)};
      }
      &:disabled {
        border: 2px solid ${transparentize(0.7, bland.decorative)};
        background-color: ${transparentize(0.9, bland.decorative)};
      }
    `,
        outlinedHasError: css `
      border: 2px solid ${danger.decorative};
      &:focus {
        border: 2px solid ${danger.decorative};
        background-color: ${transparentize(0.93, danger.decorative)};
      }
      &:not([disabled]):hover {
        border: 2px solid ${transparentize(0.3, danger.decorative)};
      }
    `,
    };
});
const TextArea = forwardRef((props, ref) => {
    var _a, _b, _c, _d, _e, _f;
    const _g = useStyles$9(props, (_a = props.component, (_a !== null && _a !== void 0 ? _a : 'textarea'))), { Root, styles, variant = 'outlined', focused: _focused, hasError: incomingHasError, disabled: incomingDisabled, onFocus, onBlur } = _g, restOfProps = __rest(_g, ["Root", "styles", "variant", "focused", "hasError", "disabled", "onFocus", "onBlur"]);
    const formControlContext = useContext(FormControlContext);
    const id = (_b = formControlContext) === null || _b === void 0 ? void 0 : _b.id;
    const hasError = (_d = (incomingHasError !== null && incomingHasError !== void 0 ? incomingHasError : (_c = formControlContext) === null || _c === void 0 ? void 0 : _c.hasError), (_d !== null && _d !== void 0 ? _d : false));
    const disabledFromFormControl = (_f = (_e = formControlContext) === null || _e === void 0 ? void 0 : _e.disabled, (_f !== null && _f !== void 0 ? _f : false));
    const disabled = incomingDisabled || disabledFromFormControl;
    const handleFocus = (e) => {
        if (onFocus) {
            onFocus(e);
        }
        if (formControlContext) {
            formControlContext.setFocused(true);
        }
    };
    const handleBlur = (e) => {
        if (onBlur) {
            onBlur(e);
        }
        if (formControlContext) {
            formControlContext.setFocused(false);
        }
    };
    return (React.createElement(Root, Object.assign({ ref: ref, id: id, disabled: disabled, className: classNames({
            [styles.outlined]: variant === 'outlined',
            [styles.outlinedHasError]: variant === 'outlined' && hasError,
            [styles.filled]: variant === 'filled',
            [styles.filledHasError]: variant === 'filled' && hasError,
        }), onFocus: handleFocus, onBlur: handleBlur }, restOfProps)));
});

/**
 * this is a fontawesome 5 free icon
 * @license https://fontawesome.com/license
 */
const CheckIcon = forwardRef((props, ref) => {
    return (React.createElement("svg", Object.assign({ ref: ref, "aria-hidden": "true", viewBox: "0 0 512 512" }, props),
        React.createElement("path", { fill: "currentColor", d: "M173.898 439.404l-166.4-166.4c-9.997-9.997-9.997-26.206 0-36.204l36.203-36.204c9.997-9.998 26.207-9.998 36.204 0L192 312.69 432.095 72.596c9.997-9.997 26.207-9.997 36.204 0l36.203 36.204c9.997 9.997 9.997 26.206 0 36.204l-294.4 294.401c-9.998 9.997-26.207 9.997-36.204-.001z" })));
});

const useStyles$a = createStyles(({ css, color, theme, surface }) => {
    const bland = createReadablePalette(theme.colors.bland, surface);
    const danger = createReadablePalette(theme.colors.danger, surface);
    const width = theme.space(2);
    const height = theme.space(2);
    const widthSmall = theme.space(1.5);
    const heightSmall = theme.space(1.5);
    const widthLarge = theme.space(2.5);
    const heightLarge = theme.space(2.5);
    return {
        root: css `
      position: relative;
      margin: ${theme.space(0.5)} 0;
      background-color: ${surface};
    `,
        hasError: css `
      color: ${danger.decorative};
      & .facade {
        border: 2px solid ${danger.decorative};
      }
      & .checkbox:focus ~ .facade {
        border: 2px solid ${danger.decorative};
        background-color: ${transparentize(0.93, danger.decorative)};
      }
      & .checkbox:not([disabled]):hover ~ .facade {
        border: 2px solid ${transparentize(0.3, danger.decorative)};
        background-color: ${transparentize(0.93, danger.decorative)};
      }
      & .checkbox:active ~ .facade {
        background-color: ${transparentize(0.9, danger.decorative)};
      }
    `,
        checkbox: css `
      cursor: pointer;
      opacity: 0;
      position: absolute;
      top: 0;
      left: 0;

      &:focus ~ .facade {
        border: 2px solid ${color.decorative};
        background-color: ${transparentize(0.93, color.decorative)};
      }
      &:hover ~ .facade {
        border: 2px solid ${transparentize(0.3, color.decorative)};
        background-color: ${transparentize(0.93, color.decorative)};
      }
      &:active ~ .facade {
        background-color: ${transparentize(0.9, color.decorative)};
      }

      &:disabled ~ .facade {
        cursor: not-allowed;
        border: 2px solid ${transparentize(0.7, bland.decorative)};
        background-color: ${transparentize(0.9, bland.decorative)};
      }

      &:disabled {
        cursor: not-allowed;
      }

      &:checked ~ .facade .icon {
        opacity: 1;
      }
    `,
        checkboxStandard: css `
      width: ${width};
      height: ${height};
    `,
        checkboxSmall: css `
      width: ${widthSmall};
      height: ${heightSmall};
    `,
        checkboxLarge: css `
      width: ${widthLarge};
      height: ${heightLarge};
    `,
        facade: css `
      width: ${width};
      height: ${height};
      display: flex;
      justify-content: center;
      align-items: center;
      flex: 0 0 auto;
      background-color: white;
      transition: border ${theme.durations.standard}ms,
        background-color ${theme.durations.standard}ms;
      border: 2px solid ${bland.decorative};
      background-color: ${surface};
    `,
        facadeSmall: css `
      width: ${widthSmall};
      height: ${heightSmall};
    `,
        facadeStandard: css `
      width: ${width};
      height: ${height};
    `,
        facadeLarge: css `
      width: ${widthLarge};
      height: ${heightLarge};
    `,
        icon: css `
      width: 1rem;
      height: 1rem;
      fill: currentColor;
      opacity: 0;
      pointer-events: none;
    `,
    };
});
const Checkbox = forwardRef((props, ref) => {
    var _a, _b, _c;
    const _d = useStyles$a(props), { Root, styles, id: incomingId, hasError: incomingHasError, disabled: incomingDisabled, onFocus, onBlur, inputRef, icon: Icon = CheckIcon, size = 'standard' } = _d, restOfProps = __rest(_d, ["Root", "styles", "id", "hasError", "disabled", "onFocus", "onBlur", "inputRef", "icon", "size"]);
    const formControlContext = useContext(FormControlContext);
    const id = (incomingId !== null && incomingId !== void 0 ? incomingId : (_a = formControlContext) === null || _a === void 0 ? void 0 : _a.id);
    const hasError = Boolean((_b = formControlContext) === null || _b === void 0 ? void 0 : _b.hasError) || Boolean(incomingHasError);
    const disabled = Boolean((_c = formControlContext) === null || _c === void 0 ? void 0 : _c.disabled) || Boolean(incomingDisabled);
    const handleFocus = (e) => {
        if (onFocus) {
            onFocus(e);
        }
        if (formControlContext) {
            formControlContext.setFocused(true);
        }
    };
    const handleBlur = (e) => {
        var _a;
        if (onBlur) {
            onBlur(e);
        }
        if (formControlContext) {
            (_a = formControlContext) === null || _a === void 0 ? void 0 : _a.setFocused(false);
        }
    };
    return (React.createElement(Root, { ref: ref, className: classNames({
            [styles.hasError]: hasError,
        }) },
        React.createElement("input", Object.assign({ type: "checkbox", id: id, ref: inputRef, className: classNames('checkbox', styles.checkbox, {
                [styles.checkboxSmall]: size === 'small',
                [styles.checkboxStandard]: size === 'standard',
                [styles.checkboxLarge]: size === 'large',
            }), onFocus: handleFocus, onBlur: handleBlur, disabled: disabled }, restOfProps)),
        React.createElement("div", { "aria-hidden": "true", className: classNames('facade', styles.facade, {
                [styles.facadeSmall]: size === 'small',
                [styles.facadeStandard]: size === 'standard',
                [styles.facadeLarge]: size === 'large',
            }) },
            React.createElement(Icon, { className: classNames('icon', styles.icon) }))));
});

var RadioGroupContext = createContext(null);

const CircleIcon = forwardRef((props, ref) => {
    return (React.createElement("svg", Object.assign({ ref: ref, "aria-hidden": "true", viewBox: "0 0 512 512" }, props),
        React.createElement("circle", { cx: "256", cy: "256", fill: "currentColor", r: "192" })));
});

const useStyles$b = createStyles(({ css, color, theme, surface }) => {
    const bland = createReadablePalette(theme.colors.bland, surface);
    const danger = createReadablePalette(theme.colors.danger, surface);
    const width = theme.space(2);
    const height = theme.space(2);
    const widthSmall = theme.space(1.5);
    const heightSmall = theme.space(1.5);
    const widthLarge = theme.space(2.5);
    const heightLarge = theme.space(2.5);
    return {
        root: css `
      position: relative;
      margin: ${theme.space(0.5)} 0;
    `,
        hasError: css `
      color: ${danger.decorative};
      & .facade {
        border: 2px solid ${danger.decorative};
      }
      & .radio:focus ~ .facade {
        border: 2px solid ${danger.decorative};
        background-color: ${transparentize(0.93, danger.decorative)};
      }
      & .radio:not([disabled]):hover ~ .facade {
        border: 2px solid ${transparentize(0.3, danger.decorative)};
        background-color: ${transparentize(0.93, danger.decorative)};
      }
      & .radio:active ~ .facade {
        background-color: ${transparentize(0.9, danger.decorative)};
      }
    `,
        radio: css `
      cursor: pointer;
      opacity: 0;
      position: absolute;
      top: 0;
      left: 0;

      &:focus ~ .facade {
        border: 2px solid ${color.decorative};
        background-color: ${transparentize(0.93, color.decorative)};
      }
      &:hover ~ .facade {
        border: 2px solid ${transparentize(0.3, color.decorative)};
        background-color: ${transparentize(0.93, color.decorative)};
      }
      &:active ~ .facade {
        background-color: ${transparentize(0.9, color.decorative)};
      }

      &:disabled ~ .facade {
        cursor: not-allowed;
        border: 2px solid ${transparentize(0.7, bland.decorative)};
        background-color: ${transparentize(0.9, bland.decorative)};
      }

      &:disabled {
        cursor: not-allowed;
      }

      &:checked ~ .facade .icon {
        opacity: 1;
      }
    `,
        radioStandard: css `
      width: ${width};
      height: ${height};
    `,
        radioSmall: css `
      width: ${widthSmall};
      height: ${heightSmall};
    `,
        radioLarge: css `
      width: ${widthLarge};
      height: ${heightLarge};
    `,
        facade: css `
      width: ${width};
      height: ${height};
      border-radius: 99999px;
      display: flex;
      justify-content: center;
      align-items: center;
      flex: 0 0 auto;
      background-color: white;
      transition: border ${theme.durations.standard}ms,
        background-color ${theme.durations.standard}ms;
      border: 2px solid ${bland.decorative};
      background-color: ${surface};
    `,
        facadeSmall: css `
      width: ${widthSmall};
      height: ${heightSmall};
    `,
        facadeStandard: css `
      width: ${width};
      height: ${height};
    `,
        facadeLarge: css `
      width: ${widthLarge};
      height: ${heightLarge};
    `,
        icon: css `
      width: 1rem;
      height: 1rem;
      fill: currentColor;
      opacity: 0;
      pointer-events: none;
    `,
    };
});
const Radio = forwardRef((props, ref) => {
    var _a, _b, _c, _d;
    const _e = useStyles$b(props), { Root, styles, id: incomingId, hasError: incomingHasError, disabled: incomingDisabled, checked: incomingChecked, name: incomingName, value, onFocus, onBlur, onChange, inputRef, icon: Icon = CircleIcon, size = 'standard' } = _e, restOfProps = __rest(_e, ["Root", "styles", "id", "hasError", "disabled", "checked", "name", "value", "onFocus", "onBlur", "onChange", "inputRef", "icon", "size"]);
    const formControlContext = useContext(FormControlContext);
    const radioGroupContext = useContext(RadioGroupContext);
    const id = (incomingId !== null && incomingId !== void 0 ? incomingId : (_a = formControlContext) === null || _a === void 0 ? void 0 : _a.id);
    const hasError = Boolean((_b = formControlContext) === null || _b === void 0 ? void 0 : _b.hasError) || Boolean(incomingHasError);
    const disabled = Boolean((_c = formControlContext) === null || _c === void 0 ? void 0 : _c.disabled) || Boolean(incomingDisabled);
    const checkedFromContext = radioGroupContext
        ? radioGroupContext.value === value
        : undefined;
    const checked = (incomingChecked !== null && incomingChecked !== void 0 ? incomingChecked : checkedFromContext);
    const name = (incomingName !== null && incomingName !== void 0 ? incomingName : (_d = radioGroupContext) === null || _d === void 0 ? void 0 : _d.name);
    const handleFocus = (e) => {
        if (onFocus) {
            onFocus(e);
        }
        if (formControlContext) {
            formControlContext.setFocused(true);
        }
    };
    const handleBlur = (e) => {
        var _a;
        if (onBlur) {
            onBlur(e);
        }
        if (formControlContext) {
            (_a = formControlContext) === null || _a === void 0 ? void 0 : _a.setFocused(false);
        }
    };
    const handleChange = (e) => {
        if (onChange) {
            onChange(e);
        }
        if (radioGroupContext) {
            radioGroupContext.handleChange(e);
        }
    };
    return (React.createElement(Root, { ref: ref, className: classNames({
            [styles.hasError]: hasError,
        }) },
        React.createElement("input", Object.assign({ type: "radio", id: id, ref: inputRef, value: value, name: name, checked: checked, className: classNames('radio', styles.radio, {
                [styles.radioSmall]: size === 'small',
                [styles.radioStandard]: size === 'standard',
                [styles.radioLarge]: size === 'large',
            }), onFocus: handleFocus, onBlur: handleBlur, onChange: handleChange, disabled: disabled }, restOfProps)),
        React.createElement("div", { "aria-hidden": "true", className: classNames('facade', styles.facade, {
                [styles.facadeSmall]: size === 'small',
                [styles.facadeStandard]: size === 'standard',
                [styles.facadeLarge]: size === 'large',
            }) },
            React.createElement(Icon, { className: classNames('icon', styles.icon) }))));
});

const useStyles$c = createStyles(({ css }) => ({
    root: css ``,
}));
const RadioGroup = forwardRef((props, ref) => {
    var _a;
    const _b = useStyles$c(props, (_a = props.component, (_a !== null && _a !== void 0 ? _a : 'div'))), { Root, styles, component, value: incomingValue, onChange } = _b, restOfProps = __rest(_b, ["Root", "styles", "component", "value", "onChange"]);
    const name = useMemo(() => `hui-group-${shortId()}`, []);
    const [stateValue, setStateValue] = useState('');
    const value = (incomingValue !== null && incomingValue !== void 0 ? incomingValue : stateValue);
    // pull the reference of onChange on invocation
    const onChangeRef = useRef(onChange);
    const handleChange = useCallback((e) => {
        const onChange = onChangeRef.current;
        if (onChange) {
            onChange(e);
        }
        setStateValue(e.currentTarget.value);
    }, []);
    const contextValue = useMemo(() => ({
        name,
        value,
        handleChange,
    }), [handleChange, name, value]);
    return (React.createElement(RadioGroupContext.Provider, { value: contextValue },
        React.createElement(Root, Object.assign({ ref: ref, role: "radiogroup" }, restOfProps))));
});

const useStyles$d = createStyles(({ css, color, theme, surface }) => {
    const bland = createReadablePalette(theme.colors.bland, surface);
    const danger = createReadablePalette(theme.colors.danger, surface);
    const width = theme.space(3.5);
    const height = theme.space(2);
    const widthSmall = theme.space(2.75);
    const heightSmall = theme.space(1.5);
    const widthLarge = theme.space(4);
    const heightLarge = theme.space(2.5);
    return {
        root: css `
      position: relative;
      margin: ${theme.space(0.5)} 0;
      background-color: ${surface};
      border-radius: 99999px;
    `,
        hasError: css `
      color: ${danger.decorative};
      & .facade {
        border: 2px solid ${danger.decorative};
      }
      & .switch:focus ~ .facade {
        border: 2px solid ${danger.decorative};
        background-color: ${transparentize(0.93, danger.decorative)};
      }
      & .switch:not([disabled]):hover ~ .facade {
        border: 2px solid ${transparentize(0.3, danger.decorative)};
        background-color: ${transparentize(0.93, danger.decorative)};
      }
      & .switch:active ~ .facade {
        background-color: ${transparentize(0.9, danger.decorative)};
      }

      & .switch:checked ~ .facade {
        background-color: ${transparentize(0.93, danger.decorative)};
      }

      & .switch:checked ~ .facade .dot {
        fill: ${danger.decorative};
      }
    `,
        switch: css `
      cursor: pointer;
      opacity: 0;
      position: absolute;
      top: 0;
      left: 0;

      &:focus ~ .facade {
        border: 2px solid ${color.decorative};
        background-color: ${transparentize(0.93, color.decorative)};
      }
      &:hover ~ .facade {
        border: 2px solid ${transparentize(0.3, color.decorative)};
        background-color: ${transparentize(0.93, color.decorative)};
      }
      &:active ~ .facade {
        background-color: ${transparentize(0.9, color.decorative)};
      }

      &:disabled ~ .facade {
        cursor: not-allowed;
        border: 2px solid ${transparentize(0.7, bland.decorative)};
        background-color: ${transparentize(0.9, bland.decorative)};
      }

      &:disabled {
        cursor: not-allowed;
      }

      &:checked ~ .facade .dot {
        cx: 125;
        fill: ${color.decorative};
      }

      &:disabled ~ .facade .dot {
        fill: ${bland.decorative};
      }

      &:checked ~ .facade {
        /* border: 2px solid ${bland.decorative}; */
        background-color: ${transparentize(0.9, color.decorative)};
      }
    `,
        switchStandard: css `
      width: ${width};
      height: ${height};
    `,
        switchSmall: css `
      width: ${widthSmall};
      height: ${heightSmall};
    `,
        switchLarge: css `
      width: ${widthLarge};
      height: ${heightLarge};
    `,
        facade: css `
      width: ${width};
      height: ${height};
      border-radius: 99999px;
      display: flex;
      justify-content: center;
      align-items: center;
      flex: 0 0 auto;
      background-color: white;
      transition: border ${theme.durations.standard}ms,
        background-color ${theme.durations.standard}ms;
      border: 2px solid ${bland.decorative};
      background-color: ${surface};
    `,
        facadeSmall: css `
      width: ${widthSmall};
      height: ${heightSmall};
    `,
        facadeStandard: css `
      width: ${width};
      height: ${height};
    `,
        facadeLarge: css `
      width: ${widthLarge};
      height: ${heightLarge};
    `,
        icon: css `
      width: ${width};
      height: ${height};
      fill: currentColor;
      pointer-events: none;
    `,
        dot: css `
      transition: cx ${theme.durations.standard}ms,
        fill ${theme.durations.standard}ms;
    `,
        dotSmall: css `
      r: 30;
    `,
        dotStandard: css `
      r: 35;
    `,
        dotLarge: css `
      r: 40;
    `,
    };
});
const Switch = forwardRef((props, ref) => {
    var _a, _b, _c;
    const _d = useStyles$d(props), { Root, styles, id: incomingId, hasError: incomingHasError, disabled: incomingDisabled, onFocus, onBlur, inputRef, size = 'standard' } = _d, restOfProps = __rest(_d, ["Root", "styles", "id", "hasError", "disabled", "onFocus", "onBlur", "inputRef", "size"]);
    const formControlContext = useContext(FormControlContext);
    const id = (incomingId !== null && incomingId !== void 0 ? incomingId : (_a = formControlContext) === null || _a === void 0 ? void 0 : _a.id);
    const hasError = Boolean((_b = formControlContext) === null || _b === void 0 ? void 0 : _b.hasError) || Boolean(incomingHasError);
    const disabled = Boolean((_c = formControlContext) === null || _c === void 0 ? void 0 : _c.disabled) || Boolean(incomingDisabled);
    const handleFocus = (e) => {
        if (onFocus) {
            onFocus(e);
        }
        if (formControlContext) {
            formControlContext.setFocused(true);
        }
    };
    const handleBlur = (e) => {
        var _a;
        if (onBlur) {
            onBlur(e);
        }
        if (formControlContext) {
            (_a = formControlContext) === null || _a === void 0 ? void 0 : _a.setFocused(false);
        }
    };
    return (React.createElement(Root, { ref: ref, className: classNames({
            [styles.hasError]: hasError,
        }) },
        React.createElement("input", Object.assign({ type: "checkbox", id: id, ref: inputRef, className: classNames('switch', styles.switch, {
                [styles.switchSmall]: size === 'small',
                [styles.switchStandard]: size === 'standard',
                [styles.switchLarge]: size === 'large',
            }), onFocus: handleFocus, onBlur: handleBlur, disabled: disabled }, restOfProps)),
        React.createElement("div", { "aria-hidden": "true", className: classNames('facade', styles.facade, {
                [styles.facadeSmall]: size === 'small',
                [styles.facadeStandard]: size === 'standard',
                [styles.facadeLarge]: size === 'large',
            }) },
            React.createElement("svg", { className: styles.icon, viewBox: "0 0 175 100" },
                React.createElement("circle", { className: classNames('dot', styles.dot, {
                        [styles.dotSmall]: size === 'small',
                        [styles.dotStandard]: size === 'standard',
                        [styles.dotLarge]: size === 'large',
                    }), cx: "50", cy: "50" })))));
});

/**
 * this is a fontawesome 5 free icon
 * @license https://fontawesome.com/license
 */
const AngleDownIcon = forwardRef((props, ref) => {
    return (React.createElement("svg", Object.assign({ ref: ref, "aria-hidden": "true", viewBox: "0 0 320 512" }, props),
        React.createElement("path", { fill: "currentColor", d: "M143 352.3L7 216.3c-9.4-9.4-9.4-24.6 0-33.9l22.6-22.6c9.4-9.4 24.6-9.4 33.9 0l96.4 96.4 96.4-96.4c9.4-9.4 24.6-9.4 33.9 0l22.6 22.6c9.4 9.4 9.4 24.6 0 33.9l-136 136c-9.2 9.4-24.4 9.4-33.8 0z" })));
});

const useStyles$e = createStyles(({ css, theme, color, surface }) => {
    const bland = createReadablePalette(theme.colors.bland, surface);
    const danger = createReadablePalette(theme.colors.danger, surface);
    return {
        root: css `
      position: relative;
      margin: ${theme.space(0.5)} 0;
    `,
        select: css `
      ${theme.fonts.body1};
      width: 100%;
      height: 100%;
      appearance: none;
      outline: none;
      border: none;
      background: none;
      padding: ${theme.space(0.75)} ${theme.space(0.5)};
      border-radius: 0;
      cursor: pointer;

      &:disabled {
        cursor: not-allowed;
      }
    `,
        selectFilled: css `
      background-color: ${transparentize(0.8, bland.decorative)};
      color: ${readableColor(surface)};
      transition: background-color ${theme.durations.standard}ms;

      &:focus {
        background-color: ${transparentize(0.92, color.decorative)};
        color: ${readableColor(surface)};
      }
      &:hover {
        background-color: ${transparentize(0.9, color.decorative)};
        color: ${readableColor(surface)};
      }
      &:disabled {
        background-color: ${transparentize(0.9, bland.decorative)};
      }
    `,
        selectFilledHasError: css `
      background-color: ${transparentize(0.9, danger.decorative)};
      color: ${readableColor(surface)};
      &:focus {
        background-color: ${transparentize(0.85, danger.decorative)};
      }
      &:not([disabled]):hover {
        background-color: ${transparentize(0.87, danger.decorative)};
      }
    `,
        selectOutlined: css `
      background-color: ${surface};
      transition: border ${theme.durations.standard}ms,
        background-color ${theme.durations.standard}ms,
        color ${theme.durations.standard}ms;
      border: 2px solid ${bland.decorative};
      color: ${readableColor(surface)};

      &:focus {
        border: 2px solid ${color.decorative};
        background-color: ${transparentize(0.93, color.decorative)};
      }
      &:hover {
        border: 2px solid ${transparentize(0.3, color.decorative)};
      }
      &:disabled {
        border: 2px solid ${transparentize(0.7, bland.decorative)};
        background-color: ${transparentize(0.9, bland.decorative)};
      }
    `,
        selectedOutlinedHasError: css `
      border: 2px solid ${danger.decorative};
      &:focus {
        border: 2px solid ${danger.decorative};
        background-color: ${transparentize(0.93, danger.decorative)};
      }
      &:not([disabled]):hover {
        border: 2px solid ${transparentize(0.3, danger.decorative)};
      }
    `,
        icon: css `
      pointer-events: none;
      width: ${theme.space(1)};
      height: ${theme.space(1)};
      position: absolute;
      right: ${theme.space(0.75)};
      top: 50%;
      transform: translateY(-50%);
    `,
    };
});
const Select = forwardRef((props, ref) => {
    var _a, _b, _c, _d;
    const formControlContext = useContext(FormControlContext);
    const _e = useStyles$e(props, (_a = props.component, (_a !== null && _a !== void 0 ? _a : 'div'))), { Root, id: incomingId, styles, component, selectRef, disabled: incomingDisabled, hasError: incomingHasError, onFocus, onBlur, variant = 'outlined' } = _e, restOfProps = __rest(_e, ["Root", "id", "styles", "component", "selectRef", "disabled", "hasError", "onFocus", "onBlur", "variant"]);
    const disabled = Boolean((_b = formControlContext) === null || _b === void 0 ? void 0 : _b.disabled) || Boolean(incomingDisabled);
    const hasError = Boolean((_c = formControlContext) === null || _c === void 0 ? void 0 : _c.hasError) || Boolean(incomingHasError);
    const id = (incomingId !== null && incomingId !== void 0 ? incomingId : (_d = formControlContext) === null || _d === void 0 ? void 0 : _d.id);
    const handleFocus = (e) => {
        if (onFocus) {
            onFocus(e);
        }
        if (formControlContext) {
            formControlContext.setFocused(true);
        }
    };
    const handleBlur = (e) => {
        if (onBlur) {
            onBlur(e);
        }
        if (formControlContext) {
            formControlContext.setFocused(false);
        }
    };
    return (React.createElement(Root, { ref: ref },
        React.createElement("select", Object.assign({ id: id, ref: selectRef, className: classNames(styles.select, {
                [styles.selectFilled]: variant === 'filled',
                [styles.selectFilledHasError]: variant === 'filled' && hasError,
                [styles.selectOutlined]: variant === 'outlined',
                [styles.selectedOutlinedHasError]: variant === 'outlined' && hasError,
            }), disabled: disabled, onFocus: handleFocus, onBlur: handleBlur }, restOfProps)),
        React.createElement(AngleDownIcon, { className: styles.icon })));
});

const useStyles$f = createStyles(({ css, theme, color }) => ({
    // button base styles
    root: css `
    ${theme.fonts.caption}
    font-weight: bold;

    display: inline-flex;
    justify-content: center;
    align-items: center;

    appearance: none;
    outline: none;
    background: transparent;
    border: none;
    text-decoration: none;
    border-radius: 999999px;
    min-height: 34px;

    &:disabled {
      cursor: not-allowed;
    }

    padding: ${theme.space(0.5)} ${theme.space(0.75)};

    & > *:not(:last-child) {
      margin-right: ${theme.space(0.5)};
    }

    & > .hui-button {
      margin-top: -${theme.space(0.5)};
      margin-bottom: -${theme.space(0.5)};
      margin-right: -${theme.space(0.75)};
      margin-left: -${theme.space(0.25)};
    }
  `,
    // variants
    outlined: css `
    border: 1px solid ${color.readable};
    color: ${color.readable};

    transition: background-color ${theme.durations.standard}ms,
      border ${theme.durations.standard}ms;

    &:disabled {
      &,
      &:hover,
      &:focus {
        color: ${transparentize(0.4, color.readable)};
        background-color: transparent;
      }
    }
  `,
    outlinedClickable: css `
    cursor: pointer;
    &:focus {
      background-color: ${transparentize(0.92, color.decorative)};
    }
    &:hover {
      background-color: ${transparentize(0.9, color.decorative)};
    }
    &:active {
      background-color: ${transparentize(0.8, color.decorative)};
    }
  `,
    filled: css `
    background-color: ${color.decorative};
    color: ${readableColor(color.decorative)};
    border: 2px solid transparent;
    transition: background-color ${theme.durations.standard}ms,
      border ${theme.durations.standard}ms;

    &:disabled {
      background-color: ${transparentize(0.5, color.decorative)};
      border: 2px solid transparent;
      color: ${transparentize(0.2, readableColor(color.decorative))};
    }
  `,
    filledClickable: css `
    cursor: pointer;
    &:focus {
      background-color: ${transparentize(0.18, color.decorative)};
      border: 2px solid ${transparentize(0.7, '#fff')};
    }
    &:hover {
      background-color: ${transparentize(0.23, color.decorative)};
    }
    &:active {
      background-color: ${transparentize(0.3, color.decorative)};
      border: 2px solid ${transparentize(0.5, '#fff')};
    }
  `,
}));
const Chip = forwardRef((props, ref) => {
    var _a, _b, _c, _d, _e, _f, _g;
    const component = ((_a = props.component, (_a !== null && _a !== void 0 ? _a : props.clickable)) ? 'button'
        : 'div');
    const _h = useStyles$f(props, component), { Root, styles, component: _component, clickable, variant = 'outlined' } = _h, restOfProps = __rest(_h, ["Root", "styles", "component", "clickable", "variant"]);
    const theme = useTheme();
    const colorContext = useColorContext();
    let color = (_d = (_b = props.color, (_b !== null && _b !== void 0 ? _b : (_c = colorContext) === null || _c === void 0 ? void 0 : _c.surface)), (_d !== null && _d !== void 0 ? _d : theme.colors.accent));
    let surface = (_g = (_e = props.surface, (_e !== null && _e !== void 0 ? _e : (_f = colorContext) === null || _f === void 0 ? void 0 : _f.surface)), (_g !== null && _g !== void 0 ? _g : theme.colors.surface));
    if (variant === 'filled') {
        surface = color;
    }
    return (React.createElement(ColorContextProvider, { surface: surface, color: color },
        React.createElement(Root, Object.assign({ className: classNames({
                [styles.outlined]: variant === 'outlined',
                [styles.outlinedClickable]: variant === 'outlined' && clickable,
                [styles.filled]: variant === 'filled',
                [styles.filledClickable]: variant === 'filled' && clickable,
            }), ref: ref }, restOfProps))));
});

const useStyles$g = createStyles(({ css, theme }) => ({
    root: css `
    width: ${theme.space(1.5)};
    height: ${theme.space(1.5)};
    margin-top: -${theme.space(0.5)};
    margin-bottom: -${theme.space(0.5)};
    margin-left: -${theme.space(0.5)};
    border-radius: 99999px;
    overflow: hidden;

    display: flex;
    align-items: center;
    justify-content: center;

    & > img {
      width: 100%;
      height: 100%;
      object-fit: cover;
    }

    & > svg {
      width: ${theme.space(1)};
      height: ${theme.space(1)};
    }
  `,
}));
const ChipThumbnail = forwardRef((props, ref) => {
    var _a;
    const _b = useStyles$g(props, (_a = props.component, (_a !== null && _a !== void 0 ? _a : 'div'))), { Root, styles, component } = _b, restOfProps = __rest(_b, ["Root", "styles", "component"]);
    return React.createElement(Root, Object.assign({ ref: ref }, restOfProps));
});

/**
 * this is a fontawesome 5 free icon
 * @license https://fontawesome.com/license
 */
const TimesIcon = forwardRef((props, ref) => {
    return (React.createElement("svg", Object.assign({ ref: ref, "aria-hidden": "true", viewBox: "0 0 352 512" }, props),
        React.createElement("path", { fill: "currentColor", d: "M242.72 256l100.07-100.07c12.28-12.28 12.28-32.19 0-44.48l-22.24-22.24c-12.28-12.28-32.19-12.28-44.48 0L176 189.28 75.93 89.21c-12.28-12.28-32.19-12.28-44.48 0L9.21 111.45c-12.28 12.28-12.28 32.19 0 44.48L109.28 256 9.21 356.07c-12.28 12.28-12.28 32.19 0 44.48l22.24 22.24c12.28 12.28 32.2 12.28 44.48 0L176 322.72l100.07 100.07c12.28 12.28 32.2 12.28 44.48 0l22.24-22.24c12.28-12.28 12.28-32.19 0-44.48L242.72 256z" })));
});

function delay(milliseconds) {
    return new Promise(resolve => setTimeout(() => resolve('delay'), milliseconds));
}

// https://usehooks.com/useDebounce/
function useDebounce(value, delay) {
    // State and setters for debounced value
    const [debouncedValue, setDebouncedValue] = useState(value);
    useEffect(() => {
        // Update debounced value after delay
        const handler = setTimeout(() => {
            setDebouncedValue(value);
        }, delay);
        // Cancel the timeout if value changes (also on delay change or unmount)
        // This is how we prevent debounced value from updating if value is changed ...
        // .. within the delay period. Timeout gets cleared and restarted.
        return () => {
            clearTimeout(handler);
        };
    }, [value, delay]);
    return debouncedValue;
}

const useStyles$h = createStyles(({ css, theme, surface }) => ({
    root: css `
    ${theme.fonts.caption};
    position: absolute;
    pointer-events: auto;
    padding: ${theme.space(0.5)} ${theme.space(1)};
    min-width: ${theme.block(1)};
    text-align: center;
    color: ${readableColor(readableColor(surface))};
    background-color: ${transparentize(0.2, readableColor(surface))};
    z-index: ${theme.zIndex.tooltip};
  `,
    top: css `
    transform: translate(-50%, -100%);
  `,
    left: css `
    transform: translate(-100%, -50%);
  `,
    bottom: css `
    transform: translate(-50%, 0);
  `,
    right: css `
    transform: translate(0, -50%);
  `,
    tooltipContainer: css `
    position: fixed;
    pointer-events: none;
    width: 100vw;
    height: 100vh;
    top: 0;
    left: 0;
  `,
}));
function Tooltip(props) {
    const { Root, styles, title, children, position = 'top' } = useStyles$h(props);
    const rootRef = useRef(null);
    const [isMouseOverRoot, setIsMouseOverRoot] = useState(false);
    const [isMouseOverTooltip, setIsMouseOverTooltip] = useState(false);
    // TODO: this would be better as one object
    const [top, setTop] = useState(0);
    const [left, setLeft] = useState(0);
    const [width, setWidth] = useState(0);
    const [height, setHeight] = useState(0);
    const margin = 8;
    let tooltipTop;
    let tooltipLeft;
    const topCenter = top + height / 2;
    const leftCenter = left + width / 2;
    if (position === 'left') {
        tooltipTop = topCenter;
        tooltipLeft = left - margin;
    }
    else if (position === 'right') {
        tooltipTop = topCenter;
        tooltipLeft = left + width + margin;
    }
    else if (position === 'bottom') {
        tooltipTop = top + height + margin;
        tooltipLeft = leftCenter;
    }
    else {
        // position === 'top'
        tooltipTop = top - margin;
        tooltipLeft = leftCenter;
    }
    const [tooltipContainer, setTooltipContainer,] = useState(null);
    const isMouseOver = isMouseOverRoot || isMouseOverTooltip;
    const debouncedMouseOver = useDebounce(isMouseOver, 500);
    // remove the tooltip container on scroll
    useEffect(() => {
        const handleScroll = () => {
            setTooltipContainer(null);
        };
        document.addEventListener('scroll', handleScroll);
        return () => {
            document.removeEventListener('scroll', handleScroll);
        };
    }, []);
    // updates the coordinates of the tooltip
    useEffect(() => {
        let element = rootRef.current;
        if (!element)
            return;
        (() => __awaiter(this, void 0, void 0, function* () {
            while (debouncedMouseOver && !!tooltipContainer && !!element) {
                element = rootRef.current;
                if (!element)
                    continue;
                const { top, left, width, height } = element.getBoundingClientRect();
                setTop(top);
                setLeft(left);
                setWidth(width);
                setHeight(height);
                yield delay(100);
            }
        }))();
    }, [debouncedMouseOver, rootRef, tooltipContainer]);
    // adds container to the DOM when the user mouses over
    useEffect(() => {
        if (!debouncedMouseOver)
            return;
        const tooltipContainer = document.createElement('div');
        tooltipContainer.classList.add(styles.tooltipContainer);
        document.body.appendChild(tooltipContainer);
        setTooltipContainer(tooltipContainer);
        return () => {
            document.body.removeChild(tooltipContainer);
            setTooltipContainer(null);
        };
    }, [debouncedMouseOver, styles]);
    const handleEnterRoot = () => {
        setIsMouseOverRoot(true);
    };
    const handleLeaveRoot = () => {
        setIsMouseOverRoot(false);
    };
    const handleTooltipEnter = () => {
        setIsMouseOverTooltip(true);
    };
    const handleTooltipLeave = () => {
        setIsMouseOverTooltip(false);
    };
    return (React.createElement(React.Fragment, null,
        children({
            ref: rootRef,
            onMouseEnter: handleEnterRoot,
            onMouseLeave: handleLeaveRoot,
        }),
        tooltipContainer &&
            createPortal(React.createElement(Root, { className: classNames({
                    [styles.top]: position === 'top',
                    [styles.bottom]: position === 'bottom',
                    [styles.left]: position === 'left',
                    [styles.right]: position === 'right',
                }), onMouseEnter: handleTooltipEnter, onMouseLeave: handleTooltipLeave, style: {
                    top: tooltipTop,
                    left: tooltipLeft,
                } }, title), tooltipContainer)));
}

const useStyles$i = createStyles(({ css, theme }) => ({
    root: css `
    max-height: 90vh;
    width: ${theme.block(8)};
    max-width: 100%;
    background-color: ${theme.colors.surface};
    color: ${readableColor(theme.colors.surface)};
    z-index: ${theme.zIndex.modal};
    margin: auto;
    overflow: hidden;
    display: flex;
    flex-direction: column;
  `,
    container: css `
    position: fixed;
    width: 100vw;
    height: 100vh;
    top: 0;
    left: 0;
    display: flex;
  `,
    backdrop: css `
    position: fixed;
    top: 0;
    left: 0;
    bottom: 0;
    right: 0;
    background-color: ${transparentize(0.5, 'black')};
    z-index: ${theme.zIndex.modal};
  `,
}));
const Modal = forwardRef((props, ref) => {
    var _a;
    const _b = useStyles$i(props, (_a = props.component, (_a !== null && _a !== void 0 ? _a : 'section'))), { Root, styles, open, 
    // TODO: in the future, we should bind to the escape button for closing
    onClose } = _b, restOfProps = __rest(_b, ["Root", "styles", "open", "onClose"]);
    const [container, setContainer] = useState(null);
    // TODO: animations would be a nice-to-have
    useEffect(() => {
        if (!open)
            return;
        const container = document.createElement('div');
        container.classList.add(styles.container);
        document.body.appendChild(container);
        setContainer(container);
        return () => {
            document.body.removeChild(container);
            setContainer(null);
        };
    }, [open, styles.container]);
    // TODO: throw something in the DOM later for SSR SEO
    return (container &&
        createPortal(React.createElement(React.Fragment, null,
            React.createElement("div", { className: styles.backdrop, onClick: onClose }),
            React.createElement(Root, Object.assign({ ref: ref }, restOfProps))), container));
});

const useStyles$j = createStyles(({ css, theme }) => ({
    root: css `
    display: flex;
    flex-direction: column;
    overflow: hidden;
    flex: 0 0 auto;
    padding: ${theme.space(1)} 0;
  `,
}));
const ModalHeader = forwardRef((props, ref) => {
    var _a;
    const _b = useStyles$j(props, (_a = props.component, (_a !== null && _a !== void 0 ? _a : 'header'))), { Root, styles, component } = _b, restOfProps = __rest(_b, ["Root", "styles", "component"]);
    return React.createElement(Root, Object.assign({ ref: ref }, restOfProps));
});

const useStyles$k = createStyles(({ css, theme }) => ({
    root: css `
    flex: 1 1 auto;
    position: relative;
    min-height: 50vh;
    overflow: hidden;
    display: flex;
    flex-direction: column;
  `,
}));
const ModalContent = forwardRef((props, ref) => {
    var _a;
    const _b = useStyles$k(props, (_a = props.component, (_a !== null && _a !== void 0 ? _a : 'div'))), { Root, styles, component } = _b, restOfProps = __rest(_b, ["Root", "styles", "component"]);
    return React.createElement(Root, Object.assign({ ref: ref }, restOfProps));
});

const useStyles$l = createStyles(({ css, theme }) => ({
    root: css `
    display: flex;
    flex-direction: column;
    overflow: hidden;
    flex: 0 0 auto;
    padding-bottom: ${theme.space(1)};
  `,
}));
const ModalFooter = forwardRef((props, ref) => {
    var _a;
    const _b = useStyles$l(props, (_a = props.component, (_a !== null && _a !== void 0 ? _a : 'footer'))), { Root, styles, component } = _b, restOfProps = __rest(_b, ["Root", "styles", "component"]);
    return React.createElement(Root, Object.assign({ ref: ref }, restOfProps));
});

const useStyles$m = createStyles(({ css, theme }) => ({
    root: css `
    display: flex;
    justify-content: flex-end;
    padding: 0 ${theme.space(1)};
    margin-top: ${theme.space(1)};
    & > *:not(:first-child) {
      margin-left: ${theme.space(1)};
    }
  `,
}));
const ModalActions = forwardRef((props, ref) => {
    var _a;
    const _b = useStyles$m(props, (_a = props.component, (_a !== null && _a !== void 0 ? _a : 'div'))), { Root, styles, component } = _b, restOfProps = __rest(_b, ["Root", "styles", "component"]);
    return React.createElement(Root, Object.assign({ ref: ref }, restOfProps));
});

const useStyles$n = createStyles(({ css, theme }) => ({
    root: css `
    height: 1em;
    line-height: 1;
    width: 1.7em;
    display: inline-flex;
    position: relative;
    overflow: visible;
  `,
    wrapper: css `
    font-size: 1.7em;
    position: absolute;
    top: 50%;
    transform: translateY(-50%);
  `,
}));
const Emoji = forwardRef((props, ref) => {
    const _a = useStyles$n(props, 'span'), { Root, styles, label, children } = _a, restOfProps = __rest(_a, ["Root", "styles", "label", "children"]);
    return (React.createElement(Root, Object.assign({ role: "img", "aria-label": label, ref: ref }, restOfProps),
        React.createElement("div", { className: styles.wrapper }, children)));
});

const useStyles$o = createStyles(({ css, theme }) => ({
    root: css `
    position: absolute;
    width: ${theme.block(2)};
    box-shadow: ${theme.shadows.subtle};
    background-color: ${theme.colors.surface};
    z-index: ${theme.zIndex.modal};
  `,
    top: css `
    transform: translate(-50%, -100%);
  `,
    left: css `
    transform: translate(-100%, -50%);
  `,
    bottom: css `
    transform: translate(-50%, 0);
  `,
    right: css `
    transform: translate(0, -50%);
  `,
    container: css `
    position: fixed;
    width: 100vw;
    height: 100vh;
    top: 0;
    left: 0;
    display: flex;
  `,
    backdrop: css `
    position: fixed;
    top: 0;
    left: 0;
    bottom: 0;
    right: 0;
    z-index: ${theme.zIndex.modal};
  `,
}));
const Menu = forwardRef((props, ref) => {
    const _a = useStyles$o(props, props.component ? props.component : 'div'), { Root, styles, anchorEl, onClose, component, position = 'bottom', children } = _a, restOfProps = __rest(_a, ["Root", "styles", "anchorEl", "onClose", "component", "position", "children"]);
    const [container, setContainer] = useState(null);
    const [boundingBox, setBoundingBox] = useState({
        top: 0,
        left: 0,
        width: 0,
        height: 0,
    });
    let top;
    let left;
    const topCenter = boundingBox.top + boundingBox.height / 2;
    const leftCenter = boundingBox.left + boundingBox.width / 2;
    // TODO: possibly add other positions:
    // ellipsis menus with icons buttons look bad
    if (position === 'left') {
        top = topCenter;
        left = boundingBox.left;
    }
    else if (position === 'right') {
        top = topCenter;
        left = boundingBox.left + boundingBox.width;
    }
    else if (position === 'bottom') {
        top = boundingBox.top + boundingBox.height;
        left = leftCenter;
    }
    else {
        top = boundingBox.top;
        left = leftCenter;
    }
    // TODO: animations would be nice
    useEffect(() => {
        if (!anchorEl)
            return;
        const container = document.createElement('div');
        container.classList.add(styles.container);
        document.body.appendChild(container);
        setContainer(container);
        const { top, left, width, height } = anchorEl.getBoundingClientRect();
        setBoundingBox({ top, left, width, height });
        return () => {
            document.body.removeChild(container);
            setContainer(null);
        };
    }, [anchorEl, styles.container]);
    return (container &&
        createPortal(React.createElement(React.Fragment, null,
            React.createElement("div", { className: styles.backdrop, onClick: onClose }),
            React.createElement(Root, Object.assign({ className: classNames({
                    [styles.top]: position === 'top',
                    [styles.bottom]: position === 'bottom',
                    [styles.left]: position === 'left',
                    [styles.right]: position === 'right',
                }), ref: ref, style: { top, left } }, restOfProps), children)), container));
});

export { Anchor, Button, CheckIcon, Checkbox, Chip, ChipThumbnail, CircleIcon, Emoji, FormControl, HelperText, Label, List, ListItem, ListItemButton, Menu, Modal, ModalActions, ModalContent, ModalFooter, ModalHeader, Radio, RadioGroup, Select, Switch, TextArea, TextInput, ThemeProvider, TimesIcon, Tooltip, createTheme, defaultTheme, useCssReset };
//# sourceMappingURL=bundle.esm.js.map
